#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_trans.h"
#include "tsync_base_transaction.h"
#include "tsync_macros.h"

#include "tsync_dcs_services.h"
#include "tsync_misc_services.h"

extern uint8_t DCS_VALUE_RECIPE[];
extern uint8_t DCS_INT_VALUE_RECIPE[];
extern uint8_t DCS_OC_HDR_RECIPE[];
extern uint8_t DCS_OC_SFI_RECIPE[];
extern uint8_t DCS_OC_LFI_RECIPE[];
extern uint8_t DCS_OC_FI_RECIPE[];

TSYNC_ERROR
TSYNC_DCS_getCardInfo(
    TSYNC_BoardHandle hnd,
    unsigned int nSlot,
    TSYNC_OptCardHdrObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct DCS_VALUE inPayload;
    inPayload.value = nSlot;

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(DCS_VALUE_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(DCS_OC_HDR_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_DCS,
        TSYNC_ID_DCS_CA_OC_INFO,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        DCS_VALUE_RECIPE,
        DCS_OC_HDR_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct DCS_OC_HDR* outPayload =
        (DCS_OC_HDR*)GetPayload(result);

    pObj->id = outPayload->id;
    pObj->ver = outPayload->ver;
    pObj->cish.pldId = outPayload->cish.pldId;
    pObj->cish.pldVer = outPayload->cish.pldVer;
    pObj->cish.rsv1 = 0;
    pObj->cish.rsv2 = 0;
    pObj->cish.num = outPayload->cish.numFeats;

    return ( err );
}

TSYNC_ERROR
TSYNC_DCS_getFeatureByIdx(
    TSYNC_BoardHandle hnd,
    TSYNC_OptCardSlotIdxObj *pInObj,
    TSYNC_OptCardFeatInstObj *pOutObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pInObj);
    CHECK_NOT_NULL(pOutObj);

    struct DCS_OC_SFI inPayload;
    inPayload.slot = pInObj->slot;
    inPayload.idx = pInObj->idx;

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(DCS_OC_SFI_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(DCS_OC_FI_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_DCS,
        TSYNC_ID_DCS_CA_OC_FEAT_IDX,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        DCS_OC_SFI_RECIPE,
        DCS_OC_FI_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct DCS_OC_FI* outPayload =
        (DCS_OC_FI*)GetPayload(result);

    pOutObj->featId = outPayload->id;
    pOutObj->inst = outPayload->inst;

    return ( err );
}

TSYNC_ERROR
TSYNC_DCS_getInstance(
    TSYNC_BoardHandle hnd,
    TSYNC_OptCardSlotFeatInstObj *pObj,
    int *nInstance)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);
    CHECK_NOT_NULL(nInstance);

    struct DCS_OC_LFI inPayload;
    inPayload.slot = pObj->slot;
    inPayload.id = pObj->featId;
    inPayload.inst = pObj->inst;

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(DCS_OC_LFI_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(DCS_INT_VALUE_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_DCS,
        TSYNC_ID_DCS_CA_INSTANCE,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        DCS_OC_LFI_RECIPE,
        DCS_INT_VALUE_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct DCS_INT_VALUE* outPayload =
        (DCS_INT_VALUE*)GetPayload(result);

    *nInstance = outPayload->value;

    return ( err );
}

TSYNC_ERROR
TSYNC_DCS_getSlot(
    TSYNC_BoardHandle hnd,
    TSYNC_OptCardFeatInstObj *pObj,
    int *nSlot)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);
    CHECK_NOT_NULL(nSlot);

    struct DCS_OC_FI inPayload;
    inPayload.id = pObj->featId;
    inPayload.inst = pObj->inst;

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(DCS_OC_FI_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(DCS_INT_VALUE_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_DCS,
        TSYNC_ID_DCS_CA_SLOT,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        DCS_OC_FI_RECIPE,
        DCS_INT_VALUE_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct DCS_INT_VALUE* outPayload =
        (DCS_INT_VALUE*)GetPayload(result);

    *nSlot = outPayload->value;

    return ( err );
}

