#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_trans.h"
#include "tsync_base_transaction.h"
#include "tsync_macros.h"

#include "tsync_gr_services.h"
#include "tsync_misc_services.h"

extern uint8_t GR_VALUE_RECIPE[];
extern uint8_t EL_VALIDITY_RECIPE[];
extern uint8_t GR_POS_LLA_RECIPE[];
extern uint8_t GR_FIX_DATA_RECIPE[];
extern uint8_t GL_SAT_INFO_RECIPE[];
extern uint8_t GR_SAT_DATA_RECIPE[];
extern uint8_t GR_MFR_MDL_RECIPE[];
extern uint8_t GR_RCVR_INFO_RECIPE[];
extern uint8_t GR_CUSTOM_MSG_RECIPE[];
extern uint8_t GR_RCVR_PARM_GET_RECIPE[];
extern uint8_t GR_RCVR_PARM_RECIPE[];
extern uint8_t GR_RCVR_PARM_SET_RECIPE[];
extern uint8_t GR_OFFSET_SET_CMD_RECIPE[];
extern uint8_t GR_POS_LLA_SET_CMD_RECIPE[];
extern uint8_t GR_RECMODE_SET_CMD_RECIPE[];
extern uint8_t GR_RECMODE_GET_CMD_RECIPE[];
extern uint8_t GR_DYNMODE_SET_CMD_RECIPE[];
extern uint8_t GR_CUSTOM_MSG_SET_CMD_RECIPE[];
extern uint8_t GR_RESET_CMD_RECIPE[];
extern uint8_t EL_REF_ID_RECIPE[];

TSYNC_ERROR
TSYNC_GR_getOffset(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int *nOffset)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(nOffset);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_OFFSET,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_VALUE_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_VALUE* outPayload =
            (GR_VALUE*)GetPayload(result);
            
        *nOffset = outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_setOffset(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int nOffset)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct GR_OFFSET_SET_CMD inPayload;
        inPayload.nInstance = nInstance;
        inPayload.nOffset = nOffset;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_OFFSET_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_OFFSET,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_OFFSET_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getValidity(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int *bTimeValid,
    int *bPpsValid)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(bTimeValid);
        CHECK_NOT_NULL(bPpsValid);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(EL_VALIDITY_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_VALIDITY,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            EL_VALIDITY_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct EL_VALIDITY* outPayload =
            (EL_VALIDITY*)GetPayload(result);
            
        *bTimeValid = outPayload->timeValid;
        *bPpsValid = outPayload->ppsValid;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getPosition(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_LLAObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_POS_LLA_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_POSITION,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_POS_LLA_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_POS_LLA* outPayload =
            (GR_POS_LLA*)GetPayload(result);
            
        pObj->lat = outPayload->lat;
        pObj->lon = outPayload->lon;
        pObj->alt = outPayload->alt;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_setPosition(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_LLAObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_POS_LLA_SET_CMD inPayload;
        inPayload.nInstance = nInstance;
        inPayload.lat = pObj->lat;
        inPayload.lon = pObj->lon;
        inPayload.alt = pObj->alt;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_POS_LLA_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_POSITION,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_POS_LLA_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getMode(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    GL_MODE *mode,
    GL_DYN  *dyn)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(mode);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_RECMODE_GET_CMD_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_RCVR_MODE,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_RECMODE_GET_CMD_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_RECMODE_GET_CMD* outPayload =
            (GR_RECMODE_GET_CMD*)GetPayload(result);
            
        *mode = outPayload->mode;
        *dyn  = outPayload->dyn;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_setMode(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    GL_MODE mode,
    GL_DYN  dyn)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct GR_RECMODE_SET_CMD inPayload;
        inPayload.nInstance = nInstance;
        inPayload.mode = mode;
        inPayload.dyn  = dyn;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_RECMODE_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_RCVR_MODE,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_RECMODE_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getDynamics(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    GL_DYN *mode)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(mode);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_DYNAMICS,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_VALUE_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_VALUE* outPayload =
            (GR_VALUE*)GetPayload(result);
            
        *mode = (GL_DYN)outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_setDynamics(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    GL_DYN mode)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct GR_DYNMODE_SET_CMD inPayload;
        inPayload.nInstance = nInstance;
        inPayload.mode = mode;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_DYNMODE_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_DYNAMICS,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_DYNMODE_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getFixData(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_FixDataObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_FIX_DATA_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_FIX_DATA,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_FIX_DATA_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_FIX_DATA* outPayload =
            (GR_FIX_DATA*)GetPayload(result);
            
        pObj->nSats = outPayload->nSats;
        pObj->pdop = outPayload->pdop;
        pObj->hdop = outPayload->hdop;
        pObj->vdop = outPayload->vdop;
        pObj->tdop = outPayload->tdop;
        pObj->fom = outPayload->fom;
        pObj->tfom = outPayload->tfom;
        pObj->herr = outPayload->herr;
        pObj->verr = outPayload->verr;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getSatData(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_SatDataObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_SAT_DATA_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_SAT_DATA,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_SAT_DATA_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_SAT_DATA* outPayload =
            (GR_SAT_DATA*)GetPayload(result);
            
        int i;
        for(i = 0; i < TSYNC_SAT_INFO_NUM; i++)
        {
            pObj->info[i].chnum = outPayload->info[i].chnum;
            pObj->info[i].svid = outPayload->info[i].svid;
            pObj->info[i].str = outPayload->info[i].str;
            pObj->info[i].bTraim = outPayload->info[i].traim;
            pObj->info[i].bInfix = outPayload->info[i].infix;
            pObj->info[i].flags = outPayload->info[i].flags;
        }
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getSurveyProg(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    unsigned int *nProgress)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(nProgress);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_SURVEY_PROG,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_VALUE_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_VALUE* outPayload =
            (GR_VALUE*)GetPayload(result);
            
        *nProgress = outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getMfrMdl(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_ManModObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_MFR_MDL_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_MFR_MDL,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_MFR_MDL_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_MFR_MDL* outPayload =
            (GR_MFR_MDL*)GetPayload(result);
            
        memset(pObj->mfr, '\0', sizeof(pObj->mfr));
        memset(pObj->mdl, '\0', sizeof(pObj->mdl));
        
        memcpy(pObj->mfr, outPayload->mfr, sizeof(outPayload->mfr));
        memcpy(pObj->mdl, outPayload->mdl, sizeof(outPayload->mdl));
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getRcvInfo(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_ReceiverInfoObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_RCVR_INFO_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_RCVR_INFO,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_RCVR_INFO_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_RCVR_INFO* outPayload =
            (GR_RCVR_INFO*)GetPayload(result);
            
        pObj->len = outPayload->len;    
            
        memset(pObj->info, '\0', sizeof(pObj->info));
        
        memcpy(pObj->info, outPayload->info, sizeof(outPayload->info));
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getCustom(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_CustomMessageObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_CUSTOM_MSG_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_CUSTOM_MSG,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_CUSTOM_MSG_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_CUSTOM_MSG* outPayload =
            (GR_CUSTOM_MSG*)GetPayload(result);
        
        pObj->len = outPayload->len;
        
        memset(pObj->msg, '\0', sizeof(pObj->msg));
        memcpy(pObj->msg, outPayload->msg, sizeof(outPayload->msg));
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_setCustom(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_CustomMessageObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_CUSTOM_MSG_SET_CMD inPayload;
        inPayload.nInstance = nInstance;
        inPayload.len = pObj->len;
        memcpy(inPayload.msg, pObj->msg, sizeof(inPayload.msg));
        
        uint16_t ctl = 0x02;//get
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_CUSTOM_MSG_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_CUSTOM_MSG,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_CUSTOM_MSG_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getNumInst(
    TSYNC_BoardHandle hnd,
    unsigned int *nInstances)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(nInstances);

        uint16_t ctl = 0x00;//get
        uint8_t  pos = 0;
        uint32_t pyldLen = 0;

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_NUM_INST,
            ctl,
            pyldLen,
            NULL,
            NULL,
            GR_VALUE_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_VALUE* outPayload =
            (GR_VALUE*)GetPayload(result);
            
        *nInstances = outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_delPos(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_DEL_POS,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            NULL,
            result,
            handle);

    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getRefId(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_RefIdObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(EL_REF_ID_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_REF_ID,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            EL_REF_ID_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct EL_REF_ID* outPayload =
            (EL_REF_ID*)GetPayload(result);
            
        memset(pObj->refid, '\0', sizeof(pObj->refid));
        memcpy(pObj->refid, outPayload->ref, sizeof(outPayload->ref));
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_reset(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    GL_RESET reset)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct GR_RESET_CMD inPayload;
        inPayload.nInstance = nInstance;
        inPayload.reset = reset;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_RESET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_RESET,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_RESET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getAntenna(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    GL_ANT_STATUS *status)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(status);

        struct GR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_ANTENNA,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_VALUE_RECIPE,
            GR_VALUE_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_VALUE* outPayload =
            (GR_VALUE*)GetPayload(result);
            
        *status = (GL_ANT_STATUS)outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_getParameter(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_ReceiverParmObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_RCVR_PARM_GET inPayload;
        inPayload.nInstance = nInstance;
        inPayload.len = pObj->len;
        memcpy(inPayload.parm, pObj->parm, sizeof(inPayload.parm));
        
        uint16_t ctl = 0x00;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_RCVR_PARM_GET_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(GR_RCVR_PARM_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_PARAMETER,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_RCVR_PARM_GET_RECIPE,
            GR_RCVR_PARM_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct GR_RCVR_PARM* outPayload =
            (GR_RCVR_PARM*)GetPayload(result);
        
        pObj->len = outPayload->len;
        
        memset(pObj->parm, '\0', sizeof(pObj->parm));
        memcpy(pObj->parm, outPayload->parm, sizeof(outPayload->parm));
        
    return ( err );
}

TSYNC_ERROR
TSYNC_GR_setParameter(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_ReceiverParmObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(pObj);

        struct GR_RCVR_PARM_SET inPayload;
        inPayload.nInstance = nInstance;
        inPayload.len = pObj->len;
        memcpy(inPayload.parm, pObj->parm, sizeof(inPayload.parm));
        
        uint16_t ctl = 0x02;//get
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(GR_RCVR_PARM_SET_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_GR,
            TSYNC_ID_GR_CA_PARAMETER,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            GR_RCVR_PARM_SET_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}
