#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    unsigned int            type;
    unsigned int            temp;
    TSYNC_ReceiverParmObj   parm;
    TSYNC_ReceiverKeyState  keyState;
    GL_COM1_USE             use;



    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: GR_GetParm <device index> <index> <type>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Parameter Message message */
    index = atoi(argv[2]);
    type  = atoi(argv[3]);

    // Clear the structure
    memset(&parm, 0, sizeof(TSYNC_ReceiverParmObj)); 

    // Convert from the Hosts (ETX) byte order to Big Endian (KTS Byte Order)
    temp = htonl(type);

    // Set the Parameter Type we want to retrieve
    memcpy(&parm.parm, &temp, sizeof(temp));
    parm.len = sizeof(temp);

    // Send Get Custom Message transaction
    err = TSYNC_GR_getParameter(hnd, index, &parm);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    // Convert from Big Endian (KTS Byte Order) to the Hosts (ETX) byte order
    memcpy(&keyState, parm.parm, sizeof(keyState)); 
    keyState.valid = ntohl(keyState.valid);
    keyState.warn  = ntohl(keyState.warn);

    // Convert from Big Endian (KTS Byte Order) to the Hosts (ETX) byte order
    memcpy(&use, parm.parm, sizeof(GL_COM1_USE));
    use = ntohl(use);

    /*
     * On definition of the environment variable SPEC_KTSAL_EN
     * to true, print out the information in the format used
     * for the equivalent set's command line.
     */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d ", type);

        switch (type)
        {
            case GL_PARM_GSSIP_KEY_STATE:
                printf("%d %d ", keyState.valid, keyState.warn);
            case GL_PARM_GSSIP_COM1_USE:
                printf("%d ", use);
                break;
            default:
                break;
        }

        printf("\n");
    }

    printf("\n");
    printf(" GR (%d) Parameter Msg:\n", (index));
    printf("  ------------------------\n");

    switch (type)
    {
        case GL_PARM_GSSIP_KEY_STATE:
            printf("Keys State:   %s\n", 
                   ((keyState.valid) ? "VALID" : "INVALID"));
            printf("Keys Warning: %s\n", 
                   ((keyState.warn)  ? "EXPIRING" : "NONE"));
            break;
        case GL_PARM_GSSIP_COM1_USE:
            {
                switch (use)
                {
                    case GL_COM1_HS_OUT:
                        printf("COM1 used for HotStart Output (%d)", use);
                        break;
                    case GL_COM1_HS_IN:
                        printf("COM1 used for HotStart Input (%d)", use);
                        break;
                    case GL_COM1_IDLE:
                        printf("COM1 Idle - not used (%d)", use);
                        break;
                    default:
                        printf("Unsupported COM1 use (%d)", use);
                        ret = 1;
                        break;
                }
            }            
            break;
        default:
            printf("Unsupported parameter type (%d)", type);
            ret = 1;
            break;
    }

    printf("\n");

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
